/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.classify;

import amil.classify.*;
import amil.types.*;
import java.util.*;
import java.text.*;

/**
 * Methods for evaluating MI learners.
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public class Evaluator {
	private static DecimalFormat DF = new DecimalFormat("####.####");
	
	/**
	 * AUROC: area under the ROC graph.
	 */
	public static double rocArea(Model m, BagList bags) {
		double[][] graph = rocGraph(m, bags);
		double area = graph[0][0] * graph[0][1];
		for (int i=1; i<graph.length; i++){
			area += (graph[i][0] - graph[i-1][0]) * 0.5 * (graph[i][1] + graph[i-1][1]);
		}
		return area;
	}	
	
	/**
	 * Return ROC plot points suitable for gnuplot input, etc.
	 */
	public static String rocPlot(Model m, BagList bags) {
		double[][] graph = rocGraph(m, bags);
		StringBuffer sb = new StringBuffer();
		sb.append("0\t0\n");
		for (int i=0; i<graph.length; i++)
			sb.append(graph[i][0]+"\t"+graph[i][1]+"\t # conf-thresh: "+graph[i][2]+"\n");
		return sb.toString();
	}
	
	/**
	 * Return 2D array containing points for the ROC graph. ([0]=FPR; [1]=TPR; [2]=THRESH)
	 */
	public static double[][] rocGraph(Model m, BagList bags) {
		Ranker[] preds = Ranker.newOrder(m.classifyBags(bags), bags.toBagArray());		
		double pos = bags.numPos();
		double neg = bags.numNeg();
		double thresh = preds[0].getScore();
		int index=0, tp=0, fp=0;
		Vector v = new Vector();
		while (thresh >= preds[preds.length-1].getScore()) {
			while (index < preds.length) {
				Bag b = (Bag)preds[index].getObject();
				if ((thresh - preds[index].getScore()) > 0 && !(b.isPositive()))
					break;
				else {
					if (b.isPositive())
						tp++;
					else
						fp++;
				}
				index++;
			}
			double[] pt = new double[3];
			pt[0] = (neg > 0) ? fp/neg : 0.0;
			pt[1] = (pos > 0) ? tp/pos : 0.0;
			pt[2] = thresh;
			v.add(pt);
			if (index < preds.length)
				thresh = preds[index].getScore();
			else
				break;
		}
		double[][] ret=new double[v.size()][3];
		for (int i=0; i<v.size(); i++)
			ret[i] = (double[])v.get(i);
		return ret;
	}
	
/*	public static double[][] rocGraph(Model m, BagList bags) {
		Ranker[] preds = Ranker.newOrder(m.classifyBags(bags), bags.toBagArray());		
		double[][] ret = new double[preds.length][3];
		int pos = bags.numPos();
		int neg = bags.numNeg();
		double tp=0.0, fp=0.0;
		for (int i=0; i<preds.length; i++) {
			Bag here = (Bag)preds[i].getObject();
			if (here.isPositive()) tp++;
			else fp++;			
			if (i<ret.length-1) {
				ret[i][0] = fp/neg;
				ret[i][1] = tp/pos;
				ret[i][2] = preds[i].getScore();
			}
		}
		ret[ret.length-1][0] = 1.0;
		ret[ret.length-1][1] = 1.0;
		ret[ret.length-1][2] = preds[ret.length-1].getScore();
		return ret;
	}	
*/

}
