/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.classify;

import amil.classify.*;
import amil.types.*;

/**
 * Abstract Model for MI Learning.
 *
 * @author Burr Settles &lt;bsettles@cs.wisc.edu&gt;
 * @version $Rev$
 */
public abstract class Model {
	CombFn combiner = null;
	double[] parameters = null;
	boolean weightBags = true;

	public void setCombFn(CombFn cf) { combiner = cf; }
	public void setParams(double[] p) { parameters = p; }

	public CombFn getCombFn() { return combiner; }
	public double[] getParams() { return parameters; }
		
	public abstract double train(BagList bags);
	
	public abstract double[] classifyBags(BagList bags, double[] params);
	public double[] classifyBags(BagList bags) {
		return classifyBags(bags, this.parameters);
	}
	
	public abstract double[][] classifyInstances(Instance[][] instances, double[] params);
	public double[][] classifyInstances(Instance[][] instances) {
		return classifyInstances(instances, this.parameters);
	}
	public double[][] classifyInstances(BagList bags, double[] params) {
		return classifyInstances(bags.toInstanceArray(), params);
	}
	public double[][] classifyInstances(BagList bags) {
		return classifyInstances(bags, this.parameters);
	}
	
	public abstract double[] gradientAt(BagList bags, double[] params, double[] scores);
	public double[] gradientAt(BagList bags, double[] params) {
		return gradientAt(bags, params, classifyBags(bags, params));
	}
	public double[] gradientAt(BagList bags) {
		return gradientAt(bags, this.parameters);
	}

	public abstract double[] scoresAt(BagList bags, double[] params);
	public double[] scoresAt(BagList bags) {
		return scoresAt(bags, this.parameters);
	}

	public abstract double valueAt(BagList bags, double[] params);
	public double valueAt(BagList bags) {
		return valueAt(bags, this.parameters);
	}
	
	public boolean weightBags() {
		return weightBags;
	}
}
