/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.classify.active;

import amil.classify.*;
import amil.types.*;
import amil.util.*;
import java.util.*;

/**
 * MI Uncertainty (MIU) instance-query strategy (Settles et. al, 2008). Query instances by their predictive uncertainty, weighted by the amount they contribute to their bags (e.g., derivative of the combining function).
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public final class InstanceMIU extends QuerySelector {
	public BagList select(Model model, BagList bags, int num) {
		BagList ret = new BagList();
		Instance[][] ilist = bags.toInstanceArray();
		double[][] preds = model.classifyInstances(bags);
		ArrayList v = new ArrayList();
		
		for (int i=0; i<ilist.length; i++) {
			// only query from pos+ bags
			if (bags.get(i).isPositive()) {
				double[] bagGradient = model.getCombFn().gradientAt(preds[i]);
				for (int j=0; j<ilist[i].length; j++) {
					// only unqueried instances
					if (!ilist[i][j].isQueried()) {
						double score = bagGradient[j] * MiscMath.gini(preds[i][j]);
/*						double score = bagGradient[j] * MiscMath.entropy(preds[i][j]);*/
						v.add(new Ranker(score, ilist[i][j]));
					}
				}				
			}
		}

		Collections.sort(v);

		for (int n=0; n<num; n++) {
			Ranker r = (Ranker)v.get(n);
			ret.add(new PseudoBag((Instance)r.getObject()));
			System.out.println("\t"+r);
		}
		return ret;
	}
}
