/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.classify.active;

import amil.classify.*;
import amil.types.*;
import java.util.*;

/**
 * Random (passive) query instance-selection strategy.
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public class InstanceRandom extends QuerySelector {
	private static Random r;
	public InstanceRandom(Random rand) {
		this.r = rand;
	}
	public InstanceRandom(int seed) {
		this(new Random(seed));
	}
	public InstanceRandom() {
		this(new Random());
	}

	public BagList select(Model model, BagList bags, int num) {
		BagList ret = new BagList();
		Bag[] blist = bags.toBagArray();
		Instance[][] ilist = bags.toInstanceArray();
		int n=0;
		while (n<num) {
			int i = r.nextInt(ilist.length);
			if (blist[i].isPositive()) {
				int j = r.nextInt(ilist[i].length);
				if (!ilist[i][j].isQueried()) {
					ret.add(new PseudoBag(ilist[i][j]));
					System.out.println("\t"+ilist[i][j]);
					n++;					
				}
			}
		}
		return ret;
	}
}
