/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.types;

import amil.types.*;


/**
 * Instance (data point).
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public class Instance {
	Bag parent; // pointer to ORIGINAL enclosing bag
	String name;
	double[] data;
	boolean target;
	boolean queried = false;
    
    public Instance(String name, double[] data, boolean target) {
//		this.bagName = bn;
		this.name = name;
		this.data = data;
		this.target = target;
		this.parent = null;
    }

    public Instance(String name, double[] data, double target) {
		this(name, data, (target > 0.5));
    }

	/**
	 * Return the feature vector as an array.
	 */
	public double[] getData() {
		return data;
	}
	
	/**
	 * Target class value (0,1).
	 */
	public double getTarget() {
/*		return (target) ? 1-1e-5 : 1e-5;*/
		return (target) ? 1.0 : 0.0;
	}
	public boolean isPositive() {
		return target;
	}
	
	/**
	 * Length of feature vector.
	 */
	public int fvLength() {
		return data.length;
	}
	
	/**
	 * Instance ID "name."
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns the parent bag this instance came from.
	 */
	public Bag getBag() {
		return parent;
	}
	
	/**
	 * Sets the parent bag for this instance.
	 */
	public void setBag(Bag parent) {
		this.parent = parent;
	}
	
	/**
	 * String representation.
	 */
	public String toString() {
		return parent.toString()+":"+name+((target) ? "+" : "-");
	}
	
	/**
	 * Has this instance been queried?
	 */
	public boolean isQueried() {
		return queried;
	}
	
	/**
	 * Set instance as queried.
	 */
	public void setQueried() {
		queried = true;
	}
	
	/**
	 * Reset query status.
	 */
	public void reset() {
		queried = false;
	}
	
	
}
