/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.types;

import amil.types.*;

/**
 * Singleton pseudo-bag use in querying instances from positive bags. Interfaces like a regular bag, but has only one instance (a query), and contains a pointer back to the original bag.
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public class PseudoBag extends Bag {
	protected Bag parent; // pointer to original bag
	
    public PseudoBag(Instance inst, boolean t) {
//		super("{"+parent.getName()+":"+inst.getName()+"}", null, t);
		super("{"+inst.getBag().getName()+":"+inst.getName()+"}", null, t);
//		super(null, null, t);
//		this.instances = new Instance[1];
//		this.instances[0] = inst;
		this.instances = new Instance[]{inst};
		this.parent = inst.getBag();
//		this.bagName = "{"+parent.getName()+":"+inst.getName()+"}";
		parent.incPseudoCount();
		inst.setQueried();
    }
    
	public PseudoBag(Instance inst) {
		this(inst, inst.isPositive());
	}
    
	/**
	 * Number of sibling bags (including this one) that have been queried.
	 */
	public int getPseudoCount() {
		return parent.getPseudoCount();
	}

	/**
	 * The weight this bag has on the objective function.
	 */
	public double getWeight() {
		return 1.0/getPseudoCount();
	}
	
	/**
	 * Stub routine.
	 */
	public void incPseudoCount() {}
	
	/**
	 * getTime
	 */
	public double getTime() {
		return parent.getTime()/parent.size();
	}
	
	
	/**
	 * Returns false (not an original bag).
	 */
	public boolean isSourceBag() {
		return false;
	}
	
	/**
	 * String representation.
	 */
	public String toString() {
		return bagName+((target) ? "+" : "-");
/*		return bagName+((target) ? "+" : "-")+"<"+getWeight()+">";*/
	}

}
