/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.types;

import java.util.*;
import java.text.*;

/**
 * Wrapper class to help score and sort/organize objects.
 *
 * @author Burr Settles &lt;&gt;
 * @version $Rev$
 */
public class Ranker implements Comparable {
	private static DecimalFormat DF = new DecimalFormat("####.######");
	private double scr;
	private Object obj;
	
    public Ranker(double score, Object object) {
        this.scr = score;
		this.obj = object;
    }

	/**
	* Comparison between two objects (for the comparable interface).
	*/
	public int compareTo(Object o) {
		Ranker r = (Ranker)o;
		if (scr < r.scr)
			return 1;
		else if (scr > r.scr)
			return -1;
		else
			return 0;
	}

	/**
	* String representation.
	*/
	public String toString() {
		return obj+"\t// "+DF.format(scr);
	}
	
	/**
	 * Love Vigilante.
	 */
	public static Ranker[] newOrder(double[] scores, Object[] objects) {
		Ranker[] ret = new Ranker[scores.length];
		for (int i=0; i<scores.length; i++)
			ret[i] = new Ranker(scores[i], objects[i]);
		Arrays.sort(ret);
		return ret;
	}
	
	/**
	 * Return the score associated with this Ranker object.
	 */
	public double getScore() {
		return scr;
	}
	
	
	/**
	 * Return the object scored/ranked by this Ranker object.
	 */
	public Object getObject() {
		return obj;
	}
	
}
