/* Copyright (C) Burr Settles, Department of Computer Sciences, UW-Madison.
This file is part of "AMIL" (Active Machine Learning Library).
http://pages.cs.wisc.edu/~bsettles/amil
This software is licensed under the terms of the GPL. For more information, 
see "LICENSE.html" included with this distribution. */

package amil.ui;

import amil.classify.active.*;
import amil.classify.*;
import amil.types.*;
import amil.util.*;
import java.text.*;
import java.util.*;
import java.io.*;

/**
 * Driver class for running MI active learning experiments with 20 Newsgroups.
 */
public class NewsGroups {
	public static final int NUM = 10;
	public static final int QSIZE = 5;
	public static final int QUERIES = 50;
	public static DecimalFormat DF = new DecimalFormat("####.####");
    
	public static void main(String[] args) {
		if (args.length != 2) {
			System.err.println("Usage: % java -cp amil.jar amil.ui.NewsGroups <querymode> <dataset>");
			System.exit(1);
		}
		
		String mode = args[0];
		String data = args[1];
		String base = data.replace(".data","");
		System.out.println("reading in data for '"+data+"'...");
		BagList bl = DataReader.readData(data);
		double[] eval = new double[NUM];
		double[][] evalAct = new double[QUERIES][NUM];
//		BagList[] blist = bl.split(new Random(711), NUM);
		
		// cycle through folds
		for (int f=0; f<NUM; f++) {
			BagList[] blist = bl.partition(base+"."+f+".fold");
			BagList trainSet = blist[1];
			BagList evalSet = blist[0];
			System.out.println("Train set: "+trainSet.size()+" ("+trainSet.numPos()+"+,"+trainSet.numNeg()+"-)");
			System.out.println("Eval set: "+evalSet.size()+" ("+evalSet.numPos()+"+,"+evalSet.numNeg()+"-)");

			Model bestM = new MILogisticRegression(0);
			double bestVal = Double.MAX_VALUE; //bestM.train(trainSet); //0;
			for (int m=0; m<10; m++) {
				Model mod = new MILogisticRegression(f*NUM+m);
				double val = mod.train(trainSet);
				if (val < bestVal) {
					bestVal = val;
					bestM = mod;
				}
			}

			eval[f] = Evaluator.rocArea(bestM, evalSet);//blist[f]);
			System.out.println("FOLD "+f+" AUROC="+DF.format(eval[f]));

			QuerySelector sel = new InstanceRandom();
			if (mode.equals("u"))
				sel = new InstanceUncertainty();				
			if (mode.equals("egl"))
				sel = new InstanceEGL();
			if (mode.equals("miu"))
				sel = new InstanceMIU();
			
			System.out.println("QUERY TYPE: "+mode+" (FOLD/REP="+f+", QSIZE="+QSIZE+", #="+QUERIES+")");
			for (int q=0; q<QUERIES; q++) {
				BagList queries = sel.select(bestM, trainSet, 5);
//				System.out.println(queries);
				trainSet.add(queries);
				double val = bestM.train(trainSet);
/*				System.out.println(DF.format(val));*/
				evalAct[q][f] = Evaluator.rocArea(bestM, evalSet);//blist[f]);
/*				System.out.println("|TrainSet| = "+trainSet.size());*/
				System.out.println("FOLD "+f+" Q "+q+" AUROC="+DF.format(evalAct[q][f]));
			}
			trainSet.reset();
		}

		System.out.println("0\t"+DF.format(MiscMath.average(eval))+"\t"+DF.format(MiscMath.confidence(eval)));
		for (int q=0; q<QUERIES; q++) {
		System.out.println(((q+1)*QSIZE)+"\t"+DF.format(MiscMath.average(evalAct[q]))+
			"\t"+DF.format(MiscMath.confidence(evalAct[q])));
		}
	}
}
